/********************************************************************
 * Gabi/main.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: main.c,v 1.6 1999/12/30 20:50:59 root Exp root $
 *
 ********************************************************************/
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <version.h>
#include <config.h>

#include <maKs.h>
#include <Packet.h>
#include <Decode.h>
#include <server_conn.h>

#include "auswert.h"
#include "handle.h"

#if defined(DB_POSTGRES) || defined (DB_MYSQL)
#include <db_util.h>
#endif

extern int    done;

FILE	*outf;
int		m_sock;
int		delflag	= 0;

#if defined(DB_POSTGRES) || defined (DB_MYSQL)
int		db_errors= 0;
int		db_save	= 0;
DB_CONN	*conn;
#endif


/********************************************************************
 *
 ********************************************************************/
void SplashScreen()
{
	fprintf(stderr,"\n");
	fprintf(stderr,"        ################################################\n");
	fprintf(stderr,"        #                  G A B I                     #\n");
	fprintf(stderr,"        #    Gebuehren Auswertungs Basis Interface     #\n");
	fprintf(stderr,"        #      Copyright 1998  by Chris Benndorf       #\n");
	fprintf(stderr,"        ################################################\n");
	fprintf(stderr,"\n");
	fflush(stderr);
}

/********************************************************************
 *
 ********************************************************************/
void quit()
{
	done++;
	close(m_sock);
}

/********************************************************************
 *
 ********************************************************************/
void usage(char *prog)
{
	SplashScreen();

	fprintf(stderr,"usage: %s [OPTIONS]\n"
		"\t[-u<USER>] Username\n"
		"\t[-p<PASS>] Password\n"
		"\t[-H<Host>] Host that runs Timm\n"
		"\t[-P<Port>] Port that Timm is running on\n"
#if defined(DB_POSTGRES) || defined (DB_MYSQL)
		"\t[-D]       Save into database OR\n"
#endif
		"\t[-o<file>] Filename to save\n"
		"\t[-r]       Delete data from Euracom after reading\n"
		"\tOutput defaults to stdout\n"
		"\t[-u,-p,-H,-P] do overwrite ~/.maKsrc\n"
		"\t%s\n",prog,rcsid);
	exit(1);
}

/********************************************************************
 *
 ********************************************************************/
int main(int argc, char **argv)
{
	extern char *optarg;
	extern int optind;
	char 		    *outfile = NULL;
	char 		    *machine = "localhost";
	char 		    *user	 = getenv("LOGNAME");
	char 		    *passwd	 = NULL;
	int			     port=TCPPORT;
	int c;

	read_maKsrc(&user, &passwd, &machine, &port);


#if defined(DB_POSTGRES) || defined (DB_MYSQL)
#define OPTSTRING	"H:P:o:u:p:Drh"
#else
#define OPTSTRING	"H:P:o:u:p:rh"
#endif
	while ((c=getopt(argc,argv,OPTSTRING)) != EOF)
		switch(c) {
			case 'P':
				port=atoi(optarg);;
				break;
			case 'u':
				user=optarg;
				break;
			case 'p':
				passwd=optarg;
				break;
			case 'H':
				machine=optarg;
				break;
			case 'o':
				outfile=optarg;
				break;
#if defined(DB_POSTGRES) || defined (DB_MYSQL)
			case 'D':
				db_save++;
				break;
#endif
			case 'r':
				delflag++;
				break;
			default:
				usage(argv[0]);
		}

#if defined(DB_POSTGRES) || defined (DB_MYSQL)
	if ((db_save && outfile))
		usage(argv[0]);
#endif

	if (!set_user(user)) usage(argv[0]);
	if (!set_passwd(passwd)) usage(argv[0]);

	signal(SIGINT,quit);
	signal(SIGHUP,quit);
	signal(SIGTERM,quit);

	InitProcs(0, stderr);

	/* Must handle Login myself */
	SetUsrProc(K_CTI_ESP, &handle_CTI_ESP);
	
#ifdef xxx
	/* Must handle Login myself */
	SetUsrProc(K_CTI_ANL, &handle_CTI_ANL);
	
	/* Need to handle K_CTI_OFF to read data*/
	SetUsrProc(K_CTI_OFF, &handle_CTI_OFF);
#endif


	if((m_sock=Server_Connect(machine,port))== -1)
		exit(1);

#if defined(DB_POSTGRES) || defined (DB_MYSQL)
	if (db_save) {
		conn=open_DB(CALLDB_NAME);
		fprintf(stdout,"Speicherung der Daten erfolgt in Datenbank \"%s\".\n",CALLDB_NAME, conn);
		fflush(stdout);
		if (!exec_DB(conn,"BEGIN")) {
			close(m_sock);
			exit_nicely(conn);
		}
	} else
#endif
	if (!outfile)
		outf = stdout;
	else 
		outf=fopen(outfile,"w+");

	conn_request(m_sock, PERM_GEBUER, AUTH_CLEARTXT, "Gabi", ASYNC);
	PacketWaitAsync(m_sock);

	if (done == 100)
			RequestData(outf);


	return(0);
}
