/********************************************************************
 * Rike/handle.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: handle.c,v 1.3 1999/12/30 20:57:53 root Exp root $
 *
 ********************************************************************/
#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

#include <maKs.h>
#include <Packet.h>
#include <Decode.h>
#include <server_conn.h>
 
extern int	done;

/********************************************************************
 *
 ********************************************************************/
void send_KFG_COMMAND(int fd, BYTE which)
{
	send_server(fd, 0, K_CTI_ANL, which, 0, NULL);
}

/********************************************************************
 *
 ********************************************************************/
void send_ORG_RNR_FA(int fd, int *last)
{
	BYTE	*data;
	int		len;

	if (get_next_kwz(&data, &len)) {
		*last = 1;
		return;
	}
	
	send_server(fd, 0, K_CTI_ORG | K_CTI_SET | K_CTI_ADD, ORG_RNR_FA, len, data);
}

/********************************************************************
 * Handle ESP group commands
 ********************************************************************/
int handle_CTI_ESP(int fd, EspMsg  *esp)
{
	switch(esp->element) {
		case ESP_CONN_REQ :
			if (!S_CTI_ERR(esp->gruppe)) {
				if (esp->data[0] == AUTH_DENY) {
					fprintf(stderr,"Login denied! %s\n", (esp->length > 1) ? (char *)&esp->data[1] : "");
					exit(1);
				} else if (esp->data[0] == AUTH_CLEARTXT) {
					conn_auth_cleartext(fd, get_user(),get_passwd(), ASYNC);
				} else if (esp->data[0] == 0) {
					send_KFG_COMMAND(fd, ANL_KFG_START);
				} else {
					fprintf(stderr,"Wrong authorization method!(0x%02X)\n",esp->data[0]);
					exit(1);
				}
			}
			return(HANDLED);
			break;
		case ESP_CONN_AUTH :
			if (!S_CTI_ERR(esp->gruppe)) {
				send_KFG_COMMAND(fd, ANL_KFG_START);
			} else {
				fprintf(stderr,"Login denied! %s\n", (esp->length > 1) ? (char *)&esp->data[0] : "");
				exit(1);
			}
			return(HANDLED);
			break;
	}
	return(NOT_HANDLED);
}

/********************************************************************
 *
 ********************************************************************/
int handle_CTI_ANL(int fd, EspMsg *esp)
{
	int	last = 0;

	switch(esp->element) {
		case ANL_KFG_START :
			if (!S_CTI_ERR(esp->gruppe)) {
				send_KFG_COMMAND(fd, ANL_KFG_DATA_START);
			}
			return(HANDLED);
			break;
		case ANL_KFG_ENDE :
			fprintf(stderr,"Download done.\nExiting.\n");
			done++;
			close(fd);
			return(HANDLED);
			break;
		case ANL_KFG_DATA_START :
			if (!S_CTI_ERR(esp->gruppe)) {
				send_ORG_RNR_FA(fd, &last);
				if (last)
					send_KFG_COMMAND(fd, ANL_KFG_DATA_ENDE);
			}
			return(HANDLED);
			break;
		case ANL_KFG_DATA_ENDE :
			if (!S_CTI_ERR(esp->gruppe)) {
				sleep(1);
				send_KFG_COMMAND(fd, ANL_KFG_ENDE);
			}
			return(HANDLED);
			break;
	}
	return(NOT_HANDLED);
}

/********************************************************************
 * Send next set of data
 ********************************************************************/
int handle_CTI_ORG(int fd, EspMsg *esp)
{
	int	last = 0;

	switch(esp->element) {
		case ORG_RNR_FA :
			send_ORG_RNR_FA(fd, &last);
			if (last)
				send_KFG_COMMAND(fd, ANL_KFG_DATA_ENDE);
			return(HANDLED);
			break;
	}
	return(NOT_HANDLED);
}

