/********************************************************************
 * Rolf/main.cpp
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: main.cpp,v 1.2 2000/01/09 19:44:18 root Exp root $
 *
 ********************************************************************/
#include <getopt.h>
#include <iostream>
#include <vector>
#include <NGApp.h>
#include <MsgHandlerNG.h>
#include <EPacket.h>

#ifndef BYTE
#define BYTE unsigned char
#endif

class Aktor {
	public:
		Aktor(int, MsgHandlerNG*);
		~Aktor() { }

		void toggle(int);
		
	protected:
		void getConfig();

		BYTE			actor;
		BYTE			function;
		BYTE			state;
		MsgHandlerNG*	mh;
	
	private:
		bool			valid;
};

Aktor::Aktor(int which, MsgHandlerNG* m) {
	actor = which;
	valid = false;
	mh = m;
	getConfig();
}

void Aktor::getConfig() {
	vector<BYTE> data(1);
	data[0] = actor;
	

	{
		ESPMsgHdr ep = mh->syncSend(MsgHandler::G_AUX, MsgHandler::AUX_AKTOR_FKT, data);
		if (ep.isError()) {
			cerr << "Fehler beim Abfragen des Aktor-Status!" << endl;
			globalApp->quit(1);
			return;
		} else {
			function = ep.data[1];
			if (function == 9)
				valid = true;
			else {
				cerr << "Der Aktor ist nicht fuer eine manuelle Schaltung konfiguriert!" << endl;
				globalApp->quit(1);
				return;
			}
		}
	}

	{
		ESPMsgHdr ep = mh->syncSend(MsgHandler::G_AUX, MsgHandler::AUX_AKTOR_MANUELL, data);
		if (ep.isError()) {
			cerr << "Fehler beim Abfragen des Aktor-Status!" << endl;
			globalApp->quit(1);
			return;
		} else {
			state = ep.data[1];
		}
	}

}

void Aktor::toggle(int s) {
	vector<BYTE> data(2);
	data[0] = actor;
	if (s == 3)
		data[1] = (state == 1) ? 2 : 1;
	else
		data[1] = (BYTE)s;
	
	if (valid) {
		{	
			ESPMsgHdr ep = mh->syncSend(MsgHandler::G_ANL, MsgHandler::ANL_KFG_START);
			if (ep.isError()) {
				cerr << "Konfigurationswunsch abgewiesen!" << endl;
				globalApp->quit(1);
				return;
			}
		}

		ESPMsgHdr ep = mh->syncSend(MsgHandler::G_AUX | K_CTI_SET, MsgHandler::AUX_AKTOR_MANUELL, data);
		if (ep.isError()) {
			cerr << "Fehler beim Schalten des Aktors!" << endl;
			globalApp->quit(1);
		}

		{
			ESPMsgHdr ep = mh->syncSend(MsgHandler::G_ANL, MsgHandler::ANL_KFG_ENDE);
			if (ep.isError()) {
				cerr << "Konfigurationswunsch abgewiesen!" << endl;
				globalApp->quit(1);
				return;
			}
		}
	}
}

void fail(MsgHandler* mh) {
	cerr << mh->failReason() << endl;
	globalApp->quit(1);
}

void usage() {
	globalApp->getOpts().printUsage(cerr);
	exit(1);
}

int main(int argc, char **argv) {
	int relais	= -1;
	int state	= -1;

	NGApp app(argc,argv);

	app.getOpts().addOption("R:","\t-R <Relais>\tzu schaltendes Relais (0/1)\n");
	app.getOpts().addOption("S:","\t-S <on|off|toggle>\tan-/aus-/umschalten\n");

	if (!app.checkOpts(argc, argv))
		usage();

	if (app.getOpts().isOption("R"))
		relais = app.getOpts().getNumOption("R");
	else
		usage();

	if (app.getOpts().isOption("S")) {
		if (!(::strncasecmp(app.getOpts().getStrOption("S"),"on",2))) state = 2;
		if (!(::strncasecmp(app.getOpts().getStrOption("S"),"of",2))) state = 1;
		if (state == -1) state = 3;
	} else
		usage();

	if (relais == -1 || state == -1)
		usage();

	MsgHandlerNG* mh = app.createMsgHandler(MsgHandler::PERM_KONFIG);

	mh->failMsg.connect(slot(&fail));
	//mh->switchDebug(true);

	if (mh->esp_conn_req()) {
		Aktor act(relais, mh);
		act.toggle(state);
	}
	return app.getState();
}

