/********************************************************************
 * auth.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: auth.c,v 1.4 2000/01/20 14:03:52 root Exp root $
 *
 ********************************************************************/
 
#include <stdio.h>
#include <time.h>
#include <netinet/in.h>

#include <glob.h>
#include <cl_utils.h>
#include <auth.h>
#include <log.h>



struct cleartext_user {
	struct cleartext_user	*next;
	char	*user;
	char	*passwd;
	LONG	addr;
	LONG	mask;
	int		rights;
};

typedef struct cleartext_user ctUser;

ctUser ULast = { NULL, NULL, NULL, 0};
ctUser *ctHead = &ULast;

/********************************************************************
 *
 ********************************************************************/
int check_ip_addr(cl_mgmt *cl, ctUser *nuser)
{
	LONG	addr;
	LONG	result;
	LONG	*ap = (LONG *)&cl->addr.sa_data[2];
	int		ret = 0;

	addr = ntohl(*ap);

	result = addr & nuser->mask;
	ret = ((addr == 0x7F000001) || (result == nuser->addr));
	
	logprintf(LOG_DEBUG,"Address-Check: 0x%08X & 0x%08X = 0x%08X %s 0x%08X (0x7F000000)\n",
			addr, nuser->mask , result, (ret)? "==" : "!=", nuser->addr);
	
	if (!ret)
		logprintf(LOG_MSG,"User %s not permitted from adress %s!\n", nuser->user, inet_ntoa(*ap));
	else
		logprintf(LOG_MSG,"User %s permitted from adress %s!\n", nuser->user, inet_ntoa(*ap));

	return(ret);
}

/********************************************************************
 *
 ********************************************************************/
unsigned char auth_method(BYTE req, char **data, int *len)
{
	*data	= 0L;
	*len	= 0;

	return(req & AUTH_IMPLEMNT);
}

/********************************************************************
 *
 ********************************************************************/
int auth_cleartext(cl_mgmt *cl, char *data)
{
	ctUser	*nuser;
	char	*name;
	char	*pw = data;
	int		ret = 0;

	name    = (char *)strsep(&pw,";");

	strcpy(cl->user, name);
	strcpy(cl->pasw, pw);
 
	nuser = ctHead;

	while (nuser->next) {
		if (!strcmp(nuser->user, cl->user)) {
			if(!strcmp(nuser->passwd, cl->pasw) || nuser->passwd[0] == '*') {
				if (cl->perm == PERM_SCTRL) {
					ret = ((nuser->rights &0x0800) ==  0x0800) && check_ip_addr(cl, nuser);
					if (ret) 
						cl->ctyp = CTI_SVRCTRL;
				} else {
					ret = ((cl->perm &  nuser->rights) == cl->perm) && check_ip_addr(cl, nuser);
					cl->ctyp = CTI_CL_APP;
				}
			}
		}
		nuser = nuser->next;
	}

	if (ret == 0 && ctHead->user[0]=='*' && ctHead->passwd[0]=='*') {
		if (cl->perm == PERM_SCTRL) {
			ret = 0;
		} else {
			cl->ctyp = CTI_CL_APP;
			ret = ((cl->perm &  ctHead->rights) == cl->perm) && check_ip_addr(cl, nuser);
		}
	}


	logprintf(LOG_MSG,"AUTH_CLEARTXT: User: %s -> %s\n",
			name, (ret) ? "ACCEPT" : "DENY");

	return(ret);
}


/********************************************************************
 *
 ********************************************************************/
int auth_encrypted(cl_mgmt *cl, char *data)
{
	int		ret = 0;
	char	*name = "unknown";

	logprintf(LOG_MSG,"AUTH_ENCRYPT: User: %s -> %s\n",
			name, (ret) ? "ACCEPT" : "DENY");

	logprintf(LOG_MSG,"AUTH_ENCRYPT not implemented yet");

	return(ret);
}


/********************************************************************
 *
 ********************************************************************/
int auth_challenge(cl_mgmt *cl, char *data)
{
	int		ret = 0;
	char	*name = "unknown";

	logprintf(LOG_MSG,"AUTH_CHALLENG: User: %s -> %s\n",
			name, (ret) ? "ACCEPT" : "DENY");

	logprintf(LOG_MSG,"AUTH_CHALLENG: not implemented yet");

	return(ret);
}


/********************************************************************
 *
 ********************************************************************/
int auth_check(cl_mgmt *cl, BYTE auth, char *data)
{
	switch( auth ) {

		case AUTH_CLEARTXT :
			return(auth_cleartext(cl, data));

		case AUTH_ENCRYPT :
			return(auth_encrypted(cl, data));

		case AUTH_CHALLENG :
			return(auth_challenge(cl, data));

		case AUTH_NONE :
			if ((cl->perm & (PERM_CTI | PERM_BMON | PERM_USER)) == cl->perm)
				return(1);
			else
				return(0);
			break;

	}
	return(0);
}

/********************************************************************
 *
 ********************************************************************/
void auth_cleartxt_read(char *ctfname)
{
	FILE	*fp	= NULL;
	ctUser	*newuser;
	char 	line[1024];
	char	user[256];
	char	passwd[256];
	uint		addr[4];
	uint		mask[4];
	uint		rights;
	
	if ((fp = fopen(ctfname, "r")) == NULL) {
		perror(ctfname);
		exit(1);
	}

	printf("\nPermissions:\nUSER\tPERM\tADDR/MASK\n");
	
	while (fgets(line,1024,fp))  {
		sscanf(line, "%s", user);
		if (!(user[0] == '#' || user[0] == '\0') && strlen(line) > 20) {
			sscanf(line, "%s %s %i %i%*c%i%*c%i%*c%i%*c%i%*c%i%*c%i%*c%i", user, passwd, &rights,
									&addr[0], &addr[1], &addr[2], &addr[3],
									&mask[0], &mask[1], &mask[2], &mask[3]);
			printf("%s	%04X	%03d.%03d.%03d.%03d/%03d.%03d.%03d.%03d\n",user,rights,
									addr[0], addr[1], addr[2], addr[3],
									mask[0], mask[1], mask[2], mask[3]);
			newuser = (ctUser *)calloc(sizeof(ctUser),1);
			if (newuser) {
				newuser->user = (char *)strdup(user);
				newuser->passwd = (char *)strdup(passwd);
				newuser->addr = ((LONG)addr[0] << 24)|((LONG)addr[1] << 16)|((LONG)addr[2] << 8)| (LONG)addr[3];
				newuser->mask = ((LONG)mask[0] << 24)|((LONG)mask[1] << 16)|((LONG)mask[2] << 8)| (LONG)mask[3];
				newuser->rights = rights;
				newuser->next = ctHead;
				ctHead = newuser;
			}
		}
	}
	fclose(fp);
}
