/********************************************************************
 * level2srvr.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: level2srvr.c,v 1.2 1999/03/07 20:02:04 root Exp $
 *
 ********************************************************************/
 
#include <stdio.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include <glob.h>
#include <cl_utils.h>

extern FILE *logfp;
int	_sock_listen = -1;

/********************************************************************
 * Open server listener
 * Parameters: Portnumber to listen on
 * Returns:    Socketnumber of listener
 ********************************************************************/
int srv_open(int port)
{
	struct sockaddr_in  serveraddr;
	int sock;
	int on = 1;

	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock == -1) {
		logerror ("Error opening socket");
		exit (1);
	}

	setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,(const char*)&on,sizeof(on));

	serveraddr.sin_family = AF_INET;
	serveraddr.sin_addr.s_addr = htonl(INADDR_ANY);
	serveraddr.sin_port = htons(port);
	if (bind (sock, (struct sockaddr *) &serveraddr, sizeof(serveraddr)) == -1) {
		logerror ("Error binding socket");
		exit (1);
	}

	_sock_listen = sock;

	listen(sock, 5);

	return(sock);
}

/********************************************************************
 * Close server listener
 * Parameters: Portnumber to listen on
 ********************************************************************/
void srv_close()
{
	close(_sock_listen);
}

/********************************************************************
 * Accept new client
 * Parameters: Socketnumber of listener port
 * Returns:    Socketnumber for new client
 ********************************************************************/
int srv_accept()
{
	return(accept(_sock_listen,(struct sockaddr *)0,(int *)0));
}

/********************************************************************
 * Accept new client
 * Parameters: Socketnumber of listener port
 * Returns:    Socketnumber for new client
 ********************************************************************/
cl_mgmt *srv_init_proc(int port)
{
	cl_mgmt *this;

	if (srv_open(port) == -1) exit(1);

	if ((this = Cl_create())==NULL) exit(1);
	Cl_init(this, _sock_listen, CTI_CL_LISTEN, "listen", "Timm");

	return(this);
}


