/********************************************************************
 * level3.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: level5.c,v 1.3 1999/12/30 21:00:15 root Exp root $
 *
 ********************************************************************/
 
#include <stdio.h>
#include <stdlib.h>
/*#include <string.h>*/
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>

#include <glob.h>
#include <analyse.h>
#include <log.h>
#include <cl_utils.h>
#include <auth.h>
#include <level3.h>
#include <level5.h>

/********************************************************************
 *
 ********************************************************************/
void l5_analyse_pack(cl_mgmt *cl, ESPMsgHdr *msg)
{
    char    *group;
    char    *command;
    int         i;

    group   = (char *)txtGrp(CTI_TYP(msg->gruppe));
    command = (char *)text(CTI_TYP(msg->gruppe), msg->element);

	if (!group)		group="NULL";
	if (!command)	group="NULL";

	logdate(LOG_L5PACKET);
    logprintf(LOG_L5PACKET,"\nLEVEL 5 Analyse:\n");
    logprintf(LOG_L5PACKET,"\tTYP   : %s [%s]\n", command, group);
    logprintf(LOG_L5PACKET,"\tAPPL  : %-13s\tUSER   : %-13s\tFD:\t%d\n", cl->appl, cl->user, cl->fd);
    logprintf(LOG_L5PACKET,"\tLINKID: ----\t\tFOLGENR: %02X\t\tANLNR : %02X\n",  msg->folgenummer,msg->anlage);
    logprintf(LOG_L5PACKET,"\tPTYP  : %02X\t\tCOMMAND: %02X\t", CTI_GRP(msg->gruppe), msg->element);
    logprintf(LOG_L5PACKET,"\tFLAGS : %s %s %s\n",
			S_CTI_RSP(msg->gruppe) ? "RSP" : "CMD",
			S_CTI_ERR(msg->gruppe) ? "ERR" : "OK ",
			showFlags(msg->gruppe));
    logprintf(LOG_L5PACKET,"\tLAENGE: %02X\t\tDATA  : ", msg->length);
    for (i=0; i < msg->length; i++)
        logprintf(LOG_L5PACKET,"%02X ", msg->data[i]);
    logprintf(LOG_L5PACKET,"\n");
}

/********************************************************************
 *
 ********************************************************************/
void *l5_new_msg(size_t len)
{
	char	*rp;

	if ((rp = calloc(len, 1)) == NULL) {
		logerror("Allocating memory");
		exit(1);
	}

	return((void *)rp);
}

/********************************************************************
 *
 ********************************************************************/
BYTE l5_inc_server_pnum()
{
	l3_inc_server_pnum();
}

/********************************************************************
 *
 ********************************************************************/
BYTE l5_linkid_req(cl_mgmt *cl)
{
	l3_linkid_req(cl);
}

/********************************************************************
 *
 ********************************************************************/
BYTE l5_send(cl_mgmt *cl, ESPMsgHdr *msg)
{
	size_t len = 5 + ((int)msg->length & 0x00FF);

	l5_analyse_pack(cl, msg);
	l3_send(cl, msg, len);
	cl->last_pak = time(NULL);
}

/********************************************************************
 *
 ********************************************************************/
void l5_conn_auth(cl_mgmt *cl, ESPMsgHdr *msg)
{
	char	buf[256];
	int		ok;


	cl->flgnr = msg->folgenummer;

	l5_analyse_pack(cl, msg);

	strncpy(buf, &msg->data[1], (((int)msg->length - 1) & 0x00FF));
	
	ok = auth_check(cl, msg->data[0], buf);

	cl->login = time(NULL);

	if (ok && cl->ctyp == CTI_SVRCTRL) {
		l5_conn_accept(cl);
		return;
	}

	if (ok) {
		l5_linkid_req(cl);
	} else {
		l5_conn_deny(cl, "Authorization failed");
		Cl_close(cl);
	}
}

/********************************************************************
 *
 ********************************************************************/
void l5_conn_accept(cl_mgmt *cl)
{
	ESPMsgHdr *answer;

	answer = (ESPMsgHdr *)l5_new_msg(sizeof(ESPMsgHdr));

	answer->folgenummer	= cl->flgnr;
	answer->anlage		= 0;
	answer->gruppe		= G_ESP | K_CTI_GET | K_CTI_ADD| K_CTI_OK| K_CTI_RSP;
	answer->element		= ESP_CONN_AUTH;
	answer->length		= 1;
	answer->data[0]		= cl->perm;

	l5_send(cl, answer);

	free(answer);
}

/********************************************************************
 *
 ********************************************************************/
void l5_srvr_stat(cl_mgmt *cl, ESPMsgHdr *req)
{
	ESPMsgHdr	*answer;
	char		*msg = "Not implemented yet!";
	int			len;
	BYTE		flags = 0;

	if (cl->ctyp != CTI_SVRCTRL) {
		Cl_close(cl);
		return;
	}

	msg = Cl_strstatus(req->data[0],req->data[1]);

	if (msg) {
		flags = K_CTI_RSP;
		len = sizeof(ESPMsgHdr) + strlen(msg) + 2;
	} else {
		flags = K_CTI_ERR| K_CTI_RSP;
		len = sizeof(ESPMsgHdr) + 2;
	}

	answer = (ESPMsgHdr *)l5_new_msg(len);

	answer->folgenummer	= req->folgenummer;
	answer->anlage		= req->anlage;
	answer->gruppe		= req->gruppe | flags; 
	answer->element		= req->element;
	answer->length		= (BYTE)((msg) ? (strlen(msg) + 1) : 1);

	if (msg) {
		strcpy(answer->data, msg);
		free(msg);
	}

	answer->data[answer->length - 1] = '\0';

	l5_send(cl, answer);

	free(answer);
}

/********************************************************************
 *
 ********************************************************************/
void l5_srvr_kill(cl_mgmt *cl, ESPMsgHdr *req)
{
	ESPMsgHdr	*answer;
	cl_mgmt		*victim;
	WORD		linkid;
	int			len;
	BYTE		flags = 0;

	l5_analyse_pack(cl, req);

	if (cl->ctyp != CTI_SVRCTRL) {
		Cl_close(cl);
		return;
	}

	linkid = req->data[0] << 8 | req->data[1];
	victim = Cl_find_by_lid(linkid);
	if (victim) {
		logdate(LOG_MSG);
    	logprintf(LOG_MSG,"Client with LINK_ID 0x%04X (%s/%s) killed by %s/%s\n",
				linkid,victim->user,victim->appl,
				cl->user,cl->appl);
		Cl_print(victim,NULL);
		if (cl->tln_res)
			l3_unregister_tln(cl);
		Cl_close(victim);
	} else {
		logdate(LOG_MSG);
    	logprintf(LOG_MSG,"%s/%s tried to kill non existing client with LINK_ID 0x%04X\n",
				linkid, cl->user, cl->appl);
		flags = K_CTI_ERR;
	}

	len = sizeof(ESPMsgHdr);

	answer = (ESPMsgHdr *)l5_new_msg(len);

	answer->folgenummer	= req->folgenummer;
	answer->anlage		= req->anlage;
	answer->gruppe		= req->gruppe | K_CTI_RSP | flags;
	answer->element		= req->element;
	answer->length		= 0;

	l5_send(cl, answer);

	free(answer);
}

/********************************************************************
 * Allen SRVR_CTRLs Aenderung signalisieren
 ********************************************************************/
void l5_srvr_wake()
{
	ESPMsgHdr	*esp;
	int			len;

	len = sizeof(ESPMsgHdr);

   	logprintf(LOG_MSG,"ESP_SRVR_WAKE len: %d\n",len);
	
	esp = (ESPMsgHdr *)l5_new_msg(len);

	esp->folgenummer= 0xFF;
	esp->anlage		= 0;
	esp->gruppe		= G_ESP | K_CTI_GET | K_CTI_ADD;
	esp->element	= ESP_SRVR_WAKE;
	esp->length		= 1;
	esp->data[0]	= 0;

	Cl_send_all_ctrl(esp, len);

	free(esp);
}


/********************************************************************
 *
 ********************************************************************/
void l5_srvr_msg(cl_mgmt *cl, BYTE code, BYTE flags, char *msg)
{
	ESPMsgHdr	*answer;
	int			len;

	len = sizeof(ESPMsgHdr) + strlen(msg) + 2;

	answer = (ESPMsgHdr *)l5_new_msg(len);

	answer->folgenummer	= cl->flgnr;
	answer->anlage		= 0;
	answer->gruppe		= G_ESP | K_CTI_GET | K_CTI_ADD| K_CTI_ERR| K_CTI_RSP;
	answer->element		= ESP_SRVR_MSG;
	answer->length		= (BYTE)(strlen(msg) + 1);
	answer->data[0]		= code;
	answer->data[1]		= flags;

	if (msg)
		strcpy(&answer->data[2], msg);


	l5_send(cl, answer);

	free(answer);
}

/********************************************************************
 *
 ********************************************************************/
void l5_conn_deny(cl_mgmt *cl, char *why)
{
	ESPMsgHdr	*answer;
	int			len;

	len = sizeof(ESPMsgHdr) + strlen(why);

	answer = (ESPMsgHdr *)l5_new_msg(len);

	answer->folgenummer	= cl->flgnr;
	answer->anlage		= 0;
	answer->gruppe		= G_ESP | K_CTI_GET | K_CTI_ADD| K_CTI_ERR| K_CTI_RSP;
	answer->element		= ESP_CONN_AUTH;
	answer->length		= (BYTE)(strlen(why) + 1);

	strcpy(&answer->data[0], why);

	l5_send(cl, answer);

	free(answer);
}

/********************************************************************
 *
 ********************************************************************/
void l5_conn_req(cl_mgmt *cl, ESPMsgHdr *msg)
{
	ESPMsgHdr	*answer;
	char		*auth_data;
	int			auth_len;
	BYTE		method;
	int			len;

	l5_analyse_pack(cl, msg);

	cl->flgnr	= msg->folgenummer;
	cl->perm	= msg->data[0];

	if (msg->length > 2) {
		strncpy(cl->appl, &msg->data[2], MAX_APPSTR_LEN);
		cl->appl[((int)msg->length & 0x00FF) - 2] = 0;
	}

	method = auth_method(msg->data[1], &auth_data, &auth_len);

	len = sizeof(ESPMsgHdr) + auth_len;

	answer = (ESPMsgHdr *)l5_new_msg(len);

	answer->folgenummer	= msg->folgenummer;
	answer->anlage		= msg->anlage;
	answer->gruppe		= msg->gruppe | K_CTI_RSP ;
	answer->element		= msg->element;
	answer->length		= auth_len + 1;
	answer->data[0]		= method;

	if (auth_len)
		memcpy(&answer->data[2], auth_data, auth_len);

	l5_send(cl, answer);

	free(answer);

	if (method == 0xFF)
		Cl_close(cl);
}

/********************************************************************
 *
 ********************************************************************/
void l5_err_response(cl_mgmt *cl, ESPMsgHdr *msg)
{
	msg->gruppe		= msg->gruppe | K_CTI_ERR ;
	l5_send(cl, msg);
}
/********************************************************************
 *
 ********************************************************************/
void l5_handle_packet(cl_mgmt *src, cl_mgmt *dst, ESPMsgHdr *msg)
{
	if (Cl_is_app(dst) || Cl_is_ctrl(dst)) {
		l5_send(dst, msg);
	} else {
		l5_analyse_pack(dst, msg);
		if (Cl_is_ctrl(src)) {
			l5_err_response(src, msg);
		} else {
			l3_send_EspMsg(src, dst, msg);
		}
	}
}
