/* ******************************************************************
 * lcrcfg.h
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: lcrcfg.h,v 1.1 2000/01/25 19:53:44 root Exp root $
 *
 * ******************************************************************/

#ifndef LCRCFG_H_
#define LCRCFG_H_

#include <fstream>
#include <iostream>
#include <strstream>
#include <iomanip>
#include <string>
#include <vector>
#include <PNumber.h>

#ifndef	BYTE
#define BYTE unsigned char
#endif

#ifndef	WORD
#define WORD unsigned short
#endif

#ifndef	LONG
#define LONG unsigned long
#endif

#ifndef	LCR_OFF
#define LCR_OFF		0x00
#define LCR_EASY	0x10
#define LCR_FULL	0x20
#endif

class provider {
	public:
		provider(const provider&);
		provider(const string&, const string&, const char* const, const int, const int = 0);
		~provider();

		const vector<BYTE>& getData() const;

		friend ostream& operator<< (ostream&, const provider&);
		
		BYTE		firma;
		string		name;
		string		comment;
		BaseNumber*	provnum;
};

class provider_list {
	public:
		provider_list();
		~provider_list();

		void add(const provider &);
		void add(const string&, const string&, const char* const, const int = 0, const int = 0);

		size_t size(const BYTE);

		// Aufbereitung des Anlagen-Datensatzes
		const vector<BYTE>& getData(const BYTE = 0, const size_t = 0) const;

		friend ostream& operator<< (ostream&, const provider_list&);

		vector<provider>	provis[2];
		vector<BYTE>		data;
};

class routing {
	public:

		routing(const routing&);
		routing(const BYTE, const WORD, const WORD, const BYTE, const BYTE, const BYTE);
		routing(const vector<BYTE>&, const size_t);
		routing(const BYTE* const);
		~routing();

		// Byte -> BCD
		BYTE bin2bcd(const WORD);

		// Aufbereitung des Anlagen-Datensatzes
		const vector<BYTE>& getData(const BYTE) const;
		
		//friend ostrstream&	operator<< ( ostrstream&, const routing&);
		friend ostream&		operator<< (ostream&, const routing&);

		BYTE	carrier;
		BYTE	minute;
		BYTE	hour;
		BYTE	wotag;
		BYTE	fbc1;
		BYTE	fbc2;
		vector<BYTE> data;
};

class routing_zone {
	public:
		routing_zone(const routing_zone&);
		routing_zone(const int);
		routing_zone(const vector<BYTE>&, const size_t);
		routing_zone(const BYTE*);
		~routing_zone();

		// Aufbereitung des Anlagen-Datensatzes
		const vector<BYTE>& getData(const BYTE) const;

		friend ostrstream&	operator<< (ostrstream&, const routing_zone&);
		friend ostream&		operator<< (ostream&, const routing_zone&);

		int				zonenum;
		vector<routing>	routes;
		vector<BYTE>	data;
};

class routing_list {
	public:
		routing_list();
		~routing_list();

		void add(const vector<BYTE>&);
		// Aufbereitung des Anlagen-Datensatzes
		const vector<BYTE>& getData(const size_t) const;

		friend ostream& operator<< (ostream&, const routing_list&);

		vector<routing_zone>	routezone;
		vector<BYTE>			data;
		BYTE					firma;
};

class zone {
	public:
		zone(const zone&);
		zone(const string&, const string&);
		zone(const char* const, const char* const);
		~zone();

		friend ostream& operator<< (ostream&, const zone&);

		string	name;
		string	comment;
};

class sperre {
	public:
		sperre(const sperre&s);
		sperre(const string&, const string&);
		~sperre();

		const vector<BYTE>& sperre::getData() const;

		friend ostream& operator<< (ostream&, const sperre&);

		BaseNumber	number;
		string	comment;
};

class sperr_list {
	public:
		sperr_list();
		~sperr_list();

		void sperr_list::add(const string&, const string&, const int = 0);

		// Aufbereitung des Anlagen-Datensatzes
		const vector<BYTE>& getData(const BYTE = 0, const size_t = 0) const;

		size_t size(const BYTE);

		friend ostream& operator<< (ostream&, const sperr_list&);

		vector<sperre>	sperren[2];
		vector<BYTE>	data;
};

class LcrConf {
	public:
	
		LcrConf(int = 0);
		~LcrConf();

		int				readConf(const string&);
		int				writeConf(const string&);
		void			printConf(ostream&);
		friend ostream& operator<< (ostream&, const LcrConf&);

		const char* getProviderName(const string&);

		int				debug;
		string			ident;
		LONG			version;	// < 3D10 = 1, 3D10 = 2
		LONG			ftrenn;		// Firmentrennung aktiv?
		LONG			lcr_aktiv;	// LCR eingeschaltet?
		LONG			lcr_easy;	// Easy (Full = 0)
		provider_list	provis;
		sperr_list		sperrliste;
		routing_list	routes[2];
		vector<ONumber>	vorwahlen;
		vector<zone>	zones;

	protected:
		BYTE				readbyte(ifstream&);
		WORD				readword(ifstream&);
		LONG				readlong(ifstream&);
		const char* const	readstring(ifstream&);
		void				writebyte(ofstream&, const BYTE);
		void				writeword(ofstream&, const WORD);
		void				writelong(ofstream&, const LONG);
		void				writestring(ofstream&, const string&);
		bool				readAgfeoConf(ifstream&);

};

#endif  LCRCFG_H_
