/* ******************************************************************
 * Tino/main.cpp
 * Part of maKs
 * Copyright Chris Benndorf, Roland Schulz, 2000
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: main.cpp,v 1.1 2000/01/25 19:53:44 root Exp root $
 *
 * ******************************************************************/

#include <iostream>
#include <vector>
#include <NGApp.h>
#include <MsgHandlerNG.h>
#include <EPacket.h>
#include "lcrcfg.h"

//#define DEBUG	1
#ifdef DEBUG
#define SEND	fake_syncSend
#else
#define SEND	syncSend
#endif

int debug = 0;

void errorExit(int err, char *cause) {
	cerr << cause << endl;
	globalApp->quit(err);
	exit(err);
}

void fail(MsgHandler* mh) {
	cerr << mh->failReason() << endl;
	globalApp->quit(1);
}

void usage() {
	globalApp->getOpts().printUsage(cerr);
	exit(1);
}

void ReadCfgPABX(MsgHandler* mh, LcrConf& cfg) {
	BYTE firmen;
	BYTE i;

	if (mh->esp_conn_req()) {
		{
			if (debug)
				cout << "Firmenanzahl lesen..." << endl;
			ESPMsgHdr ep = mh->SEND(MsgHandler::G_ORG | K_CTI_GET, MsgHandler::ORG_FA_ANZAHL);
			if (ep.isError()) 
				errorExit(1,"Fehler beim Lesen der Firmenanzahl!");
			firmen = ep.data[0];
		}

		{
			if (debug)
				cout << "LCR-Funktion lesen..." << endl;
			for(i = 0; i < firmen; i++) {
				vector<BYTE> data(1);
				data[0] = i;		// Firma
				ESPMsgHdr ep = mh->SEND(MsgHandler::G_ORG | K_CTI_GET, MsgHandler::ORG_FA_LCR_SELECT, data);
				if (ep.isError()) 
					errorExit(1,"Fehler beim Lesen des LCR-Schalters!");

				switch (ep.data[1]) {
					case 0:
						cfg.lcr_aktiv	= 0;
						cfg.lcr_easy	= 0;
						break;
					case 0x10:
						cfg.lcr_aktiv	= 1;
						cfg.lcr_easy	= 1;
						break;
					case 0x20:
						cfg.lcr_aktiv	= 1;
						cfg.lcr_easy	= 0;
						break;
				}
				
			}
		}

		{
			if (debug)
				cout << "Vorwahlen lesen";
			for (i = 0; i < 250;) {
				vector<BYTE> data(2);
				data[0] = 0;
				data[1] = i;
				ESPMsgHdr ep = mh->SEND(MsgHandler::G_ORG | K_CTI_GET, MsgHandler::ORG_LCR_ONKZ, data);
				if (ep.isError()) 
					errorExit(1,"Fehler beim Lesen des LCR-Schalters!");

				if (debug) {
					cout << '.';
					cout.flush();
				}

				if (ep.data.size() == 2) 
					break;
				
				vector<BYTE>::iterator cur = ep.data.begin();
				cur++; 
				cur++; // index ueberspringen

				while (cur != ep.data.end()) {
					vector<BYTE> number;
					BYTE	zone	= (*cur++);
					size_t	len		= (size_t)(*cur++);
					copy(cur, cur + len, back_inserter(number));
					cur += len;
					ONumber x(number, zone);
					cfg.vorwahlen.push_back(x);
					number.clear();
					i++;
				}
			}
			if (debug)
				cout << " fertig" << endl;
		}

		if (debug)
			cout << "Provider-Daten lesen..." << endl;
		int idx = 0;
		BYTE firma = 0;
		while( 1 ) {
			if ((idx == 10) && (firmen > 1)) {
				idx = 0;
				firma = 1;
			}
			// 20 Provider, 10 pro Firma bei Firmentrennung
			vector<BYTE> data(4);
			data[0] = PNumber::PROVIDER;
			data[1] = firma;
			data[2] = idx >> 8;
			data[3] = (idx & 0x00FF);
			ESPMsgHdr ep = mh->SEND(MsgHandler::G_ORG | K_CTI_GET, MsgHandler::ORG_RNR_FA, data);
			if (ep.isError()) 
				errorExit(1,"Fehler beim Lesen der Provider-Daten!");

			vector<BYTE>::iterator cur = ep.data.begin();
			cur++; // Speicherklasse
			cur++; // Firma
			cur++; 
			cur++; // index ueberspringen

			while (cur != ep.data.end()) {
				vector<BYTE> number;
				size_t	len	= (size_t)(*cur++);
				copy(cur, cur + len, back_inserter(number));
				cur += len;
				BaseNumber x(number);
				const string name = cfg.getProviderName(x.Phone());
				cfg.provis.add(name, string(""), x.Phone().c_str(), idx++, (int)firma);
			}
			if ((idx == 20) || (firma > 1)) break;
		}

		if (debug)
			cout << "LCR-Sperrnummern lesen..." << endl;
		idx = 0;
		firma = 0;
		i = 2;
		while( i-- ) {
			if (!i) {
				if (firmen > 1) {
					idx = 0;
					firma = 1;
				} else {
					idx = 10;
				}
			}
			// 20 Sperrnummern, 10 pro Firma bei Firmentrennung
			vector<BYTE> data(4);
			data[0] = PNumber::LCRSPERRE;
			data[1] = firma;
			data[2] = idx >> 8;
			data[3] = (idx & 0x00FF);
			ESPMsgHdr ep = mh->SEND(MsgHandler::G_ORG | K_CTI_GET, MsgHandler::ORG_RNR_FA, data);
			if (ep.isError()) 
				errorExit(1,"Fehler beim Lesen der LCR-Sperrnummern!");

			vector<BYTE>::iterator cur = ep.data.begin();
			cur++; // Speicherklasse
			cur++; // Firma
			cur++; 
			cur++; // index ueberspringen

			while (cur != ep.data.end()) {
				vector<BYTE> number;
				size_t	len	= (size_t)(*cur++);
				copy(cur, cur + len, back_inserter(number));
				cur += len;
				BaseNumber x(number);
				cfg.sperrliste.add(x.Phone(), string(""), (int)firma);
			}
			if (!i) break;
		}

		{	// 2 Firmen a 20 Routing-Zonen
			if (debug) {
				cout << "Routing-Daten lesen";
				cout.flush();
			}
			for (size_t f = 0; f < 2; f++) {
				for (size_t j = 0; j < 20; j++) {
					vector<BYTE> data(2);
					data[0] = f;
					data[1] = j;
					ESPMsgHdr ep = mh->SEND(MsgHandler::G_ORG | K_CTI_GET, MsgHandler::ORG_FA_LCR_PROFIL, data);
					if (ep.isError()) 
						errorExit(1,"Fehler beim Lesen der Routing-Daten!");
					if (debug) {
						cout << '.';
						cout.flush();
					}
					cfg.routes[f].add(ep.data);
				}
			}
			if (debug)
				cout << " fertig" << endl << endl;
		}
		
		for (i = 0; i < 20; i++) {
			switch (i) {
				case 0:
					cfg.zones.push_back(zone("Ortsnetz",""));
					break;
				case 6:
					cfg.zones.push_back(zone("National",""));
					break;
				case 16:
					cfg.zones.push_back(zone("International",""));
					break;
				default:
					cfg.zones.push_back(zone("Unbekannt",""));
					break;
			}
		}
	}

	cfg.version = 2;
}

void WriteCfgPABX(MsgHandler* mh, const LcrConf& cfg) {
	int noS		= globalApp->getOpts().isOption('S');
	int noV		= globalApp->getOpts().isOption('V');
	int doE		= globalApp->getOpts().isOption('E');

#ifndef DEBUG
	if (mh->esp_conn_req()) {

		mh->config_start(true);
#endif

		if (doE) {
			if (debug)
				cout << "LCR-Funktion einschalten..." << endl;
			vector<BYTE> data(2);
			data[0] = 0;		// Firma
			data[1] = LCR_FULL;	// 0x00 = Aus, 0x10 = Easy, 0x20 = Full
			ESPMsgHdr ep = mh->SEND(MsgHandler::G_ORG | K_CTI_SET, MsgHandler::ORG_FA_LCR_SELECT, data);
			if (ep.isError()) 
				errorExit(1,"Fehler beim Schreiben des LCR-Schalters!");
		}

		{	// 20 Provider, 10 pro Firma bei Firmentrennung
			if (debug)
				cout << "Provider-Daten schreiben..." << endl;
			ESPMsgHdr ep = mh->SEND(MsgHandler::G_ORG | K_CTI_SET, MsgHandler::ORG_RNR_FA, cfg.provis.getData());
			if (ep.isError()) 
				errorExit(1,"Fehler beim Schreiben der Provider-Daten!");

			if (!cfg.ftrenn) 
				ep = mh->SEND(MsgHandler::G_ORG | K_CTI_SET, MsgHandler::ORG_RNR_FA, cfg.provis.getData(0,10));
			else
				ep = mh->SEND(MsgHandler::G_ORG | K_CTI_SET, MsgHandler::ORG_RNR_FA, cfg.provis.getData(1));
			if (ep.isError()) 
				errorExit(1,"Fehler beim Schreiben der Provider-Daten!");
		}

		if (!noS) {	// 20 Sperrnummern, 10 pro Firma bei Firmentrennung
			if (debug)
				cout << "LCR-Sperrnummern schreiben..." << endl;
			ESPMsgHdr ep = mh->SEND(MsgHandler::G_ORG | K_CTI_SET, MsgHandler::ORG_RNR_FA, cfg.sperrliste.getData());
			if (ep.isError()) 
				errorExit(1,"Fehler beim Schreiben der LCR-Sperrnummern!");

			if (!cfg.ftrenn) 
				ep = mh->SEND(MsgHandler::G_ORG | K_CTI_SET, MsgHandler::ORG_RNR_FA, cfg.sperrliste.getData(0,10));
			else
				ep = mh->SEND(MsgHandler::G_ORG | K_CTI_SET, MsgHandler::ORG_RNR_FA, cfg.sperrliste.getData(1));

			if (ep.isError()) 
				errorExit(1,"Fehler beim Schreiben der LCR-Sperrnummern!");
		}

		if (!noV) {	// Vorwahlnummern
			if (debug)
				cout << "Vorwahlnummern loeschen..." << endl;
			// Vorwahlen loeschen
			ESPMsgHdr ep = mh->SEND(MsgHandler::G_ORG | K_CTI_DEL, MsgHandler::ORG_LCR_ONKZ);
			if (ep.isError()) 
				errorExit(1,"Fehler beim Loeschen der Vorwahlnummern!");

			// und neu eintragen
			if (debug) {
				cout << "Vorwahlnummern schreiben";
				cout.flush();
			}
			vector<BYTE> data; 
			vector<const ONumber>::iterator cur = cfg.vorwahlen.begin();
			while (cur != cfg.vorwahlen.end()) {
				while (data.size() < 0x70) {
					if (cur == cfg.vorwahlen.end())
						break;
					data.push_back((*cur).getZone());
					const vector<BYTE> x = (*cur).NumberData(); 
					data.push_back((BYTE)x.size());
					copy(x.begin(), x.end(), back_inserter(data));
					cur++;
				}

				ep = mh->SEND(MsgHandler::G_ORG | K_CTI_ADD, MsgHandler::ORG_LCR_ONKZ, data);
				if (ep.isError()) 
					errorExit(1,"Fehler beim Schreiben der Vorwahlnummern!");

				if (debug) {
					cout << '.';
					cout.flush();
				}

				data.clear();
			}
			if (debug)
				cout << " fertig" << endl;
		}

		{	// 2 Firmen a 20 Routing-Zonen
		  for (size_t f = 0; f < 2; f++) {
		    if (debug)
		      cout << "Routing loeschen (" << f+1 << ") ..." << endl;
		    ESPMsgHdr ep = mh->SEND(MsgHandler::G_ORG | K_CTI_DEL,
					    MsgHandler::ORG_FA_LCR_PROFIL, f);
		    if (ep.isError()) 
		      errorExit(1,"Fehler beim Loeschen der Routing Daten!");
		    if (debug) {
		      cout << "Routing-Daten schreiben (" << f+1 << ") ";
		      cout.flush();
		    }
		    for (size_t j = 0; j < cfg.routes[0].routezone.size();j++)
		      {
			ESPMsgHdr ep = mh->SEND(MsgHandler::G_ORG | K_CTI_SET,
						MsgHandler::ORG_FA_LCR_PROFIL,
						cfg.routes[f].getData(j));
			
			if (ep.isError()) 
			  errorExit(1, "Fehler beim Schreiben der Routing-Daten!");
			if (debug) {
			  cout << '.';
			  cout.flush();
			}
			
		      }
		    if (debug)
		      cout << " fertig" << endl;
		  }
		}
		
#ifndef DEBUG
		if (debug)
			cout << "Konfiguration sichern..." << endl;
		if (mh->config_end(true) && debug)
			cout << "Konfiguration beendet!" << endl;
	}
#endif
}

void ReadCfgFile(LcrConf& cfg, string& lcrfile) {
	if (cfg.readConf(lcrfile))
		errorExit(1,"Fehler beim Lesen der LCR-Datei!");
	
	if (cfg.debug) {
		cfg.printConf(cout);
	}
	
	if (cfg.vorwahlen.size() > 250) {
		cerr << "ACHTUNG, DIE ANZAHL DER VORWAHLEINTRAEGE UEBERSCHREITET DAS MAXIMUM VON 250!!!" << endl;
		cerr << "Uebertragung abgebrochen! Bitte bearbeiten Sie diese LCR-Datei vor der Uebertragung!" << endl;
		exit(1);
	}
}

int main(int argc, char **argv) {
	string lcrfile;

	NGApp app(argc,argv);

	app.getOpts().addOption("w:","\t[-w <LCR-Datei>]\tDatei -> Anlage\n");
	app.getOpts().addOption("r:","\t[-r <LCR-Datei>]\tAnlage -> Datei\n");
	app.getOpts().addOption("a:","\t[-a <LCR-Datei>]\tNur analysieren\n");
	app.getOpts().addOption("S","\t[-S]\tSperrnummern nicht ueberschreiben\n");
	app.getOpts().addOption("V","\t[-V]\tVorwahlnummern nicht ueberschreiben\n");
	app.getOpts().addOption("E","\t[-E]\tLCR immer aktivieren\n");
	app.getOpts().addOption("v","\t[-v[v]...[v]] (verbose++)\n");

	if (!app.checkOpts(argc, argv))
		usage();

	if (app.getOpts().isOption("r"))
		lcrfile = app.getOpts().getStrOption("r");

	if (app.getOpts().isOption("w"))
		lcrfile = app.getOpts().getStrOption("w");

	if (app.getOpts().isOption("a"))
		lcrfile = app.getOpts().getStrOption("a");

	debug		= app.getOpts().isOption('v');
	int	do_a	= app.getOpts().isOption('a');
	int	do_R	= app.getOpts().isOption('r');
	int	do_W	= app.getOpts().isOption('w');

	if (lcrfile.empty() || (do_R && do_W) || (do_a && do_W) || (do_a && do_R))
		usage();

	LcrConf cfg(debug);

	MsgHandlerNG* mh = app.createMsgHandler(MsgHandler::PERM_KONFIG);

	mh->failMsg.connect(slot(&fail));
	mh->switchDebug(false);

	if (do_R) {
		ReadCfgPABX(mh, cfg);
		cfg.writeConf(lcrfile);
	}

	if (do_W) {
		cfg.readConf(lcrfile);
		WriteCfgPABX(mh, cfg);
	}

	if (do_a) {
		cfg.readConf(lcrfile);
		debug++;
	}

	if (debug)
		cfg.printConf(cout);

	return app.getState();
}


