/* ******************************************************************
 * AnlageParm.cpp
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: AnlageParm.cpp,v 1.5 1999/12/30 21:01:36 root Exp $
 *
 * ******************************************************************/

#include <stdio.h>
#include <vector>
#include <iostream>

#include <AnlageParm.h>

Anlage::Anlage()
{
}

Anlage::~Anlage()
{
}

ostream& operator<< ( ostream& strm , const Anlage& anlg)
{
	strm <<
		anlg.n_type << " " <<
		(int)anlg.n_bart << " " <<
		(int)anlg.n_nab << " " <<
		(int)anlg.n_ntfe << " " <<
		(int)anlg.n_s0 << " " <<
		(int)anlg.n_s0ext << " " <<
		(int)anlg.n_s0int << " " <<
		(int)anlg.n_s0quer << " " <<
		(int)anlg.n_bytes << endl;
	strm << anlg.s_type << endl ;
	strm << anlg.s_vers.Version << endl;
	return strm;
}

void Anlage::setTyp(const vector<BYTE> &typ)
{
	// Kopieren _ohne_ '\0' !!!
	s_type.assign(typ.begin(), typ.end() - 1);

	if (s_type == "26X")
		n_type = 260 + (n_s0 - 2);
	else if (s_type == "18X")
		n_type = 180 + (n_s0 - 1);
	else
		n_type = 141;
 
}

void Anlage::setMaxParm(const vector<BYTE>& parm)
{
	n_nab = parm[0];
	n_ntfe = parm[1];
	n_s0 = parm[2];
	n_s0ext = parm[3];
	n_s0int = parm[4];
	n_s0quer = parm[5];
	n_bytes = parm[6];
}

void Anlage::setSWVers(const vector<BYTE>& _vers)
{
	memcpy(&s_vers,_vers.begin(), sizeof(AnlSwVersType));
}

void Anlage::setBArt(BYTE a)
{
	n_bart = a;
}

int Anlage::S0Sum() const
{
	return((int)n_s0);
}

int Anlage::S0Int() const
{
	return((int)n_s0int);
}

int Anlage::S0Ext() const
{
	return((int)n_s0ext);
}

int Anlage::EvtBytes() const
{
	return((int)n_bytes);
}

int Anlage::Typ() const
{
	return(n_type);
}

char *Anlage::Vers() const
{
	static char dummy[50];

	sprintf(dummy,"%s (%d.%d.%d.%d)", s_vers.Version, s_vers.Rel4, s_vers.Rel3, s_vers.Rel2, s_vers.Rel1);

	return(dummy);
}

