/* ******************************************************************
 * EPacket.cpp
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: EPacket.cpp,v 1.4 2000/01/08 10:18:27 root Exp root $
 *
 * ******************************************************************/
#include <iomanip>
#include <cctype>
#include <unistd.h>
#include <EPacket.h>

ESPMsgHdr::ESPMsgHdr(BYTE flgn, BYTE anlg, BYTE grp, BYTE elem,
		     const vector<BYTE> &buf)
{
	folgenummer = flgn;
	anlage      = anlg;
	gruppe      = grp;
	element     = elem;
	data        = buf;
}

ESPMsgHdr::ESPMsgHdr(const BYTE *buf)
{
        folgenummer = buf[0];
	anlage      = buf[1];
	gruppe      = buf[2];
	element     = buf[3];

	int len = buf[4];
	copy(&buf[5], &buf[5]+len,back_inserter(data));
}

void ESPMsgHdr::read( int fd )
{	
  BYTE buf[256];
  int len;

  ::read(fd, buf, 5);
  
  folgenummer      = buf[0];
  anlage           = buf[1];
  gruppe           = buf[2];
  element          = buf[3];
  
  len = (int)(buf[4] & 0x00FF);

  ::read(fd, buf, len);

  copy(buf, buf+len, back_inserter(data));
}

void ESPMsgHdr::write( int fd )
{
  BYTE buf[5];

  buf[0] = folgenummer;
  buf[1] = anlage;
  buf[2] = gruppe;
  buf[3] = element;
  buf[4] = data.size();
  
  ::write(fd, buf, 5);
  ::write(fd, data.begin(), data.size());
}

bool ESPMsgHdr::isError() {
	return ((gruppe & 0x04) == 0x04);
}

bool ESPMsgHdr::isAnswer() {
	return ((gruppe & 0x08) == 0x08);
}

ostream& operator<< ( ostream& strm , const ESPMsgHdr& ep)
{
  strm << hex << setfill('0') << 
    "FLG: " <<  setw(2) << (int)ep.folgenummer	<<
    " ANL: " << setw(2) << (int)ep.anlage		<<
    " TYP: " << setw(2) << (int)ep.gruppe		<<
    " CMD: " << setw(2) << (int)ep.element		<<
    " LEN: " << setw(2) << (int)ep.data.size()	<<
    " DAT: " ;
  
  for (const BYTE* bp = ep.data.begin(); bp < ep.data.end(); bp++)
    strm << setw(2) << (int)*bp << ' ';
  
  strm << dec << setfill(' ');
  return strm;
}
