/* ******************************************************************
 * MsgHandler.cpp
 * Part of maKs
 * Copyright Chris Benndorf und Roland Schulz, 1998, 1999,2000
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: MsgHandler.cpp,v 1.6 2000/01/03 21:12:44 root Exp $
 *
 * ******************************************************************/
 
#include "EPacket.h"
#include "MsgHandler.h"

/********************************************************************
 *
 ********************************************************************/
MsgHandler::MsgHandler(const char *app, const char *user, const char *passw, const BYTE auth, const BYTE perm) :
	_auth(auth), _perm(perm)
{
	_flgnr		= 0;
	_fail		= "noErr";
	_fatal		= 0;
	warte		= 0;
	debug		= false;
	connected	= false;

	strncpy(_app, app,   32);
	strncpy(_usr, user,  32);
	strncpy(_psw, passw, 32);
}

/********************************************************************
 *
 ********************************************************************/
BYTE MsgHandler::fnum()
{
	_flgnr++;

	if (_flgnr >= 0x7F) 
		_flgnr = 0;

	return(_flgnr);
}

/********************************************************************
 *
 ********************************************************************/

void MsgHandler::fake_asyncSend(BYTE grp, BYTE elem, 
			   const vector<BYTE> &buf, BYTE anlg)
{
    ESPMsgHdr np(fnum(), anlg, grp, elem, buf);
    if (debug)
    	cout << "fakeSend: " << np << endl;
}

/********************************************************************
 *
 ********************************************************************/

const ESPMsgHdr& MsgHandler::fake_syncSend(BYTE grp, BYTE elem,
			  const vector<BYTE> &buf, BYTE anlg) 
{
	wp = ESPMsgHdr(fnum(), anlg, grp, elem, buf);
    if (debug)
		cout << "fakeSend: " << wp << endl;
	return wp;
}

/********************************************************************
 *
 ********************************************************************/

void MsgHandler::asyncSend(BYTE grp, BYTE elem, 
			   const vector<BYTE> &buf, BYTE anlg)
{
    ESPMsgHdr np(fnum(), anlg, grp, elem, buf);
    if (debug)
		cout << "asyncSend: " << np << endl;
    np.write(socket());
}

/********************************************************************
 *
 ********************************************************************/
const ESPMsgHdr& MsgHandler::syncSend(BYTE grp, BYTE elem,
			  const vector<BYTE> &buf, BYTE anlg) 
{
	wp = ESPMsgHdr(fnum(), anlg, grp, elem, buf);
	if (debug)
		cout << "syncSend : " << wp << endl;
	wp.write(socket());

	warte = true;
	retry = 0;
	while (retry<20 && warte==true)
		processEvents();

	if (warte==false) {
		return wp;
	}
	else {
		if (debug)
			cout << "syncSend : FEHLER! Angefordertes Packet nicht empfangen!";
		warte = false;
		wp = ESPMsgHdr();
		return wp;
	}
}

/********************************************************************
 *
 ********************************************************************/

void MsgHandler::Handle()
{
	ESPMsgHdr	ep;
	
	ep.read(socket());

	if (debug) 
		cout << "Handle(" << warte << "): " << ep << endl;
	if ( warte && ((ep.gruppe & 0xF0) == (wp.gruppe & 0xF0)) && (ep.element == wp.element)) {
		wp = ep;
		warte = false;
	} else {
		retry++;
		msg_Emitter(&ep);
	}
}

/********************************************************************
 *
 ********************************************************************/
void MsgHandler::switchDebug(bool _state)
{
	if (_state != debug)
		debug = _state;
}

/********************************************************************
 *
 ********************************************************************/
bool MsgHandler::esp_conn_req() 
{
    vector<BYTE> buf;
    buf.push_back(_perm);
    buf.push_back(_auth);
    copy(_app, _app+strlen(_app), back_inserter(buf));
    buf.push_back(0x00);

    ESPMsgHdr esp = syncSend(0, ESP_CONN_REQ, buf);
    if (esp.data.size() == 0)
		sendfailMsg("Verbindung fehlgeschlagen",true);
    else if (esp.data[0] == 0xFF)
		sendfailMsg((char *)&esp.data[1]);
    else if (esp.data[0] == 0x00) {
		connected = true;
		sendrdyMsg(&esp);
		return true;
	}
    else
		return esp_conn_auth();
	
	return false;
}

/********************************************************************
 *
 ********************************************************************/
bool MsgHandler::esp_conn_auth() 
{
    vector<BYTE> buf;

    buf.push_back(_auth);
    copy(_usr, _usr+strlen(_usr), back_inserter(buf));
    buf.push_back(';');
    copy(_psw, _psw+strlen(_psw), back_inserter(buf));
    buf.push_back(0x00);

    ESPMsgHdr esp = syncSend(0, ESP_CONN_AUTH, buf);
    if (esp.isError()) {
		connected = false;
   		sendfailMsg((char *)&esp.data[1]);
		return false;
    } else {
		connected = true;
		sendrdyMsg(&esp);
		return true;
	}
}

/********************************************************************
 *
 ********************************************************************/
bool MsgHandler::config_start(bool dodata) 
{
    ESPMsgHdr esp = syncSend(G_ANL, ANL_KFG_START);
    if (esp.isError()) {
		sendfailMsg("ANL_KFG_START abgewiesen");
		return false;
    } 
	if (dodata) {
    	esp = syncSend(G_ANL, ANL_KFG_DATA_START);
    	if (esp.isError()) {
			sendfailMsg("ANL_KFG_DATA_START abgewiesen");
			return false;
    	} 
	}
	return true;
}

/********************************************************************
 *
 ********************************************************************/
bool MsgHandler::config_end(bool dodata) 
{
	ESPMsgHdr esp;
	if (dodata) {
    	esp = syncSend(G_ANL, ANL_KFG_DATA_ENDE);
    	if (esp.isError()) {
			sendfailMsg("ANL_KFG_DATA_ENDE abgewiesen");
			return false;
    	} 
	}
    esp = syncSend(G_ANL, ANL_KFG_ENDE);
    if (esp.isError()) {
		sendfailMsg("ANL_KFG_ENDE abgewiesen");
		return false;
    } 
	return true;
}

