/* ******************************************************************
 * MsgHandler.cpp
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: MsgHandlerQt.cpp,v 1.2 2000/01/08 10:18:27 root Exp root $
 *
 * ******************************************************************/
 
#include <qapplication.h>

#include "MsgHandlerQt.moc"

/********************************************************************
 *
 ********************************************************************/
MsgHandlerQt::MsgHandlerQt(const char *host, const int port, const char *app, const char *user, const char *passwd, const BYTE auth, const BYTE perm) :
	MsgHandler(app, user, passwd, auth, perm)
{
	conn = new KSocket(host, port);

	if ( socket() == -1) 
		sendfailMsg("Could not connect to server",true);

	connect	(	conn,  SIGNAL( readEvent(KSocket*) ),
				this,	SLOT( slotHandle() )
			);

	connect	(	conn, SIGNAL( closeEvent(KSocket*) ),
				this,	SLOT( slotClose() )
			);

	conn->enableRead(true);
}

/********************************************************************
 *
 ********************************************************************/
void MsgHandlerQt::slotClose()
{
	conn->enableRead(false);
	sendfailMsg("Server closed connection",true);
	exit(1);
}

/********************************************************************
 *
 ********************************************************************/
void MsgHandlerQt::slotHandle()
{
  Handle();
}

/********************************************************************
 *
 ********************************************************************/
void MsgHandlerQt::msg_Emitter(ESPMsgHdr *ep)
{
	switch ((ep->gruppe & 0xF0)) {
		case G_ESP :
		  emit espMsg(ep);
		  break;
		case G_ANL :
		  emit anlMsg(ep);
		  break;
		case G_POR :
		  emit porMsg(ep);
		  break;
		case G_GEB :
		  emit gebMsg(ep);
		  break;
		case G_ORG :
		  emit orgMsg(ep);
		  break;
		case G_TLN :
		  emit tlnMsg(ep);
		  break;
		case G_AUX :
		  emit auxMsg(ep);
		  break;
		case G_STA :
		  emit staMsg(ep);
		  break;
		case G_OFF :
		  emit offMsg(ep);
		  break;
		case G_VBG :
		  emit vbgMsg(ep);
		  break;
		case G_EVT :
		  emit evtMsg(ep);
		  break;
		case G_TST :
		  emit tstMsg(ep);
		  break;
		case G_FAB :
		  emit fabMsg(ep);
		  break;
	}
}

/********************************************************************
 *
 ********************************************************************/
void MsgHandlerQt::sendfailMsg(char *msg, int fatal) 
{
	_fail = msg;
	_fatal = fatal;
	emit failMsg(this);
}

void MsgHandlerQt::processEvents()
{
  qApp->processEvents();
}




