#include "NGApp.h"
#include <fstream>
#include <cstdlib>
#include <algorithm>
#include <functional>
#include <getopt.h>
#include <cstdio>

NGApp* globalApp;

NGApp::NGApp(int argc, char** argv ) {
	globalApp		= this;
	ret				= 0;
	done			= false;
	tset.tv_sec		= 0;
	tset.tv_usec	= 100;
	mh				= NULL;

	name = argv[0];
	name.erase(0,name.rfind('/') + 1);

	string s = "usage:\t" + name + "\n";
	opts.initUsage(s);

	readConf();
}

bool NGApp::checkOpts(int argc, char** argv) {
	opts.addOption("H:","\t[-H<hostname>]   \tHost that runs Timm\n");
	opts.addOption("P:","\t[-P<port number>]\tPort that Timm is running on\n");
	opts.addOption("u:","\t[-u<username>]   \tUsername\n");
	opts.addOption("p:","\t[-p<password>]   \tPassword\n");
	opts.addUsage(      "\t[-u,-p,-H,-P]    \tdo overwrite ~/.maKsrc\n");

	opts.parseOptions(argc, argv);

	if (opts.isOption("H"))
		host = opts.getStrOption("H");
	if (opts.isOption("P"))
		port = opts.getNumOption("P");
	if (opts.isOption("u"))
		user = opts.getStrOption("u");
	if (opts.isOption("p"))
		pass = opts.getStrOption("p");

	return true;
}

void NGApp::quit(int retval) {
	ret		= retval;
	done	= true;
}

void NGApp::readConf() {
	string maxrc = std::getenv("HOME");
	maxrc += "/.maKsrc";

	std::ifstream cfg(maxrc.c_str());

	if (cfg) {
		string token;
		while(!cfg.eof()) {
			cfg >> token;
			if ((token[0] == '#')) {
				getline(cfg,token); 
			} else {
				if (!(::strncasecmp(token.c_str(),"user",4)))	cfg >> user;
				if (!(::strncasecmp(token.c_str(),"pass",4)))	cfg >> pass;
				if (!(::strncasecmp(token.c_str(),"host",4)))	cfg >> host;
				if (!(::strncasecmp(token.c_str(),"port",4)))	cfg >> port;
			}
		}
	}
}

string& NGApp::getAppName() {
	return name;
}

string& NGApp::getUser() {
	return user;
}

string& NGApp::getPass() {
	return pass;
}

string& NGApp::getHost() {
	return host;
}

int NGApp::getPort() {
	return port;
}

void NGApp::connect(NGNotifier* n) {
	switch (n->getTyp()) {
		case NGNotifier::READ :
			rvec.push_back(n);	break;

		case NGNotifier::WRITE :
			wvec.push_back(n);	break;

		case NGNotifier::EXCEPTION :
			evec.push_back(n);	break;
	}
	n->setConnected(true);
}

void NGApp::disconnect(NGNotifier* n) {
	switch (n->getTyp()) {
		case NGNotifier::READ :
			rvec.erase(remove_if(rvec.begin(), rvec.end(), bind2nd(equal_to<NGNotifier*>(), n)),rvec.end());
			break;

		case NGNotifier::WRITE :
			wvec.erase(remove_if(wvec.begin(), wvec.end(), bind2nd(equal_to<NGNotifier*>(), n)),wvec.end());
			break;

		case NGNotifier::EXCEPTION :
			wvec.erase(remove_if(wvec.begin(), wvec.end(), bind2nd(equal_to<NGNotifier*>(), n)),wvec.end());
			break;
	}
}

MsgHandlerNG* NGApp::createMsgHandler(BYTE perm) {
	mh = new MsgHandlerNG(
						host.c_str(),
						port,
						name.c_str(),
						user.c_str(),
						pass.c_str(),
						MsgHandler::AUTH_CLEARTXT,
						perm
					);

	return mh;
}

MsgHandlerNG* NGApp::registerMsgHandler(MsgHandlerNG* m) {
	mh = m;
	return mh;
}


void NGApp::setMaxWait(int uSecs) {
	tset.tv_sec		= uSecs / 1000;
	tset.tv_usec 	= uSecs - ( tset.tv_sec * 1000 );
}

void NGApp::processEvents() {
	struct timeval		tout;
	if ((!rvec.empty()) || (!wvec.empty()) || (!evec.empty())) {

		int biggest = 0;
		fd_set* rfds;
		fd_set* wfds;
		fd_set* efds;

		// Clear sets
		FD_ZERO(&rset);
		FD_ZERO(&wset);
		FD_ZERO(&eset);

		// set set bits
		if (!rvec.empty()) {
			vector<NGNotifier*>::iterator cur = rvec.begin();
			while(cur != rvec.end()) {
				if ((*cur)->decriptor() > biggest) biggest = (*cur)->decriptor();
				FD_SET((*cur)->decriptor(), &rset);
				cur++;
			}
			rfds = &rset;
		} else {
			rfds = NULL;
		}

		if (!wvec.empty()) {
			vector<NGNotifier*>::iterator cur = wvec.begin();
			while(cur != wvec.end()) {
				if ((*cur)->decriptor() > biggest) biggest = (*cur)->decriptor();
				FD_SET((*cur)->decriptor(), &wset);
				cur++;
			}
			wfds = &wset;
		} else {
			wfds = NULL;
		}

		if (!evec.empty()) {
			vector<NGNotifier*>::iterator cur = evec.begin();
			while(cur != evec.end()) {
				if ((*cur)->decriptor() > biggest) biggest = (*cur)->decriptor();
				FD_SET((*cur)->decriptor(), &eset);
				cur++;
			}
			efds = &eset;
		} else {
			efds = NULL;
		}

		
		biggest += 1;
		tout.tv_sec		= tset.tv_sec;
		tout.tv_usec	= tset.tv_usec;

		// do select
		select(biggest, rfds, wfds, efds, &tout);

		// check for events
		if (rfds) {
			vector<NGNotifier*>::iterator cur = rvec.begin();
			while(cur != rvec.end()) {
				int tfd = (*cur)->decriptor();
				if (FD_ISSET(tfd, &rset)) {
					(*cur)->activated.emit(tfd);
				}
				cur++;
			}
		} else {
		}

		if (wfds) {
			vector<NGNotifier*>::iterator cur = wvec.begin();
			while(cur != wvec.end()) {
				if (FD_ISSET((*cur)->decriptor(), &wset)) {
					(*cur)->activated.emit((*cur)->decriptor());
				}
				cur++;
			}
		} else {
		}

		if (efds) {
			vector<NGNotifier*>::iterator cur = evec.begin();
			while(cur != evec.end()) {
				if (FD_ISSET((*cur)->decriptor(), &eset)) {
					(*cur)->activated.emit((*cur)->decriptor());
				}
				cur++;
			}
		} else {
		}
		
//
// sollte man hier noch irgendeine IDLE-routine aufrufen???
//

	} else {
		// Wenn wir hier ankommen, will das Programm _nichts_ tun...
		ret = 0;
		done = true;
	}
}

int NGApp::exec() {
	while(!done) {
		processEvents();
	}
	return ret;
}

clopt::clopt(char o, bool n) {
	opt		= o;
	hasparm	= n;
	flag	= 0;
}

clopt::clopt(const clopt& c) {
	opt		= c.opt;
	hasparm	= c.hasparm;
	flag	= 0;
}

void clopt::setValue(char* s) {
	value = s;
}

void clopt::setValue(string& s) {
	value = s;
}

void clopt::setFlag() {
	flag++;
}

int clopt::isSet() {
	if (hasparm && value.empty())
		return(0);

	return flag;
}

void Options::addOption(char* what, char* usgmsg, bool needparm) {
	if (isOption(what) != -1) {
		cerr << "Duplicate option \"" << what << "\" ignored!" << endl;
		return;
	}

	opts	+= what;
	usage.append(usgmsg);
	if ((what[1] == ':') || (needparm)) {
		clopt c(what[0], true);
		data.push_back(c);
	} else {
		clopt c(what[0], false);
		data.push_back(c);
	}
}

void Options::addUsage(char* s) {
	usage.append(s);
}

void Options::addUsage(string& s) {
	usage.append(s);
}

int Options::isOption(char search) {
	vector<clopt>::iterator cur = data.begin();

	while(cur != data.end()) {
		if ((*cur).opt == search)
			return (*cur).isSet();
		cur++;
	}
	return -1;
}

int Options::isOption(char* search) {
	return isOption(search[0]);
}

int Options::getNumOption(char search) {
	int ret = -1;
	vector<clopt>::iterator cur = data.begin();
	while(cur != data.end()) {
		if ((*cur).opt == search) {
			::sscanf((*cur).value.c_str(),"%i",&ret);
			return ret;
		}
		cur++;
	}
	return ret;
}

int Options::getNumOption(char* search) {
	return getNumOption(search[0]);
}

const char* Options::getStrOption(char search) {
	vector<clopt>::iterator cur = data.begin();
	while(cur != data.end()) {
		if ((*cur).opt == search) {
			return (*cur).value.c_str();
		}
		cur++;
	}
	return NULL;
}

const char* Options::getStrOption(char* search) {
	return getStrOption(search[0]);
}

void Options::printUsage(ostream& s) {
	s << usage;
}

bool Options::parseOptions(int argc, char** argv) {
	extern char *optarg;
	//extern int optind;
	int c;

	while ((c=getopt(argc, argv, opts.c_str())) != EOF) {
		vector<clopt>::iterator cur = data.begin();
		while(cur != data.end()) {
			if ((*cur).opt == c) {
				if ((*cur).hasparm)
					(*cur).setValue(optarg);
				(*cur).setFlag();
			}
			cur++;
		}
	}
	return true;
}


