/* ******************************************************************
 * NGApp.h
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: NGApp.h,v 1.2 2000/01/09 12:29:56 root Exp root $
 *
 * ******************************************************************/

#ifndef NGAPP_H_
#define NGAPP_H_

#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <vector>
#include <string>

#include <NGNotifier.h>
#include <MsgHandlerNG.h>

class clopt {
	public:
		clopt(char, bool);
		clopt(const clopt &);
		~clopt() {}

		int		isSet();
		void	setFlag();
		void	setValue(char*);
		void	setValue(string&);
		
		char	opt;
		int		flag;
		bool	hasparm;
		string	value;
};

class Options {
	public:
		Options()	{}
		~Options()	{}

		void	addOption(char*, char*, bool = false);
		int		isOption(char);
		int		isOption(char*);
		int		getNumOption(char);
		int		getNumOption(char*);
		const char*	getStrOption(char);
		const char*	getStrOption(char*);

		void	printUsage(ostream&);
		void	addUsage(char* s);
		void	addUsage(string& s);
		void	initUsage(char* s)		{ usage = s; }
		void	initUsage(string& s)	{ usage = s; }

		bool	parseOptions(int, char**);

	protected:
		string			opts;
		string			usage;
		vector<clopt>	data;
};

class NGApp {
	public:
		NGApp(int, char**);
		~NGApp() 							{ if (mh) delete mh; }

		void			quit(int = 0);
		int				exec();
		void			setMaxWait(int);
		void			processEvents();

		void			connect(NGNotifier*);
		void			disconnect(NGNotifier*);

		MsgHandlerNG*	createMsgHandler(BYTE);
		MsgHandlerNG*	registerMsgHandler(MsgHandlerNG*);
		MsgHandlerNG*	getMsgHandler()		{ return mh; }

		void			readConf();
		bool			checkOpts(int, char**);
		Options&		getOpts() { return opts; }
		string&			getUser();
		string&			getPass();
		string&			getHost();
		string&			getAppName();
		int				getPort();
		
		int				getState() 			{ return ret; }

	protected:
		bool				done;
		int					ret;
		Options				opts;

		string				name;
		string				user;
		string				pass;
		string				host;
		int					port;

		fd_set				rset;
		fd_set				wset;
		fd_set				eset;

		struct timeval		tset;

		vector<NGNotifier*>	rvec;
		vector<NGNotifier*>	wvec;
		vector<NGNotifier*>	evec;

		MsgHandlerNG*			mh;
};

extern NGApp* globalApp;

#endif NGAPP_H_
