/* ******************************************************************
 * TlnInfo.cpp
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: TlnInfo.cpp,v 1.4 2000/01/03 21:12:44 root Exp $
 *
 * ******************************************************************/

#include <stdio.h>
#include <string>
#include <iostream>
#include <vector>

#include <TlnInfo.h>

TlnInfo::TlnInfo(const char* data)
{
	b1		= data[0];
	b2		= data[1];
	typ		= data[2];
	utyp	= data[3];
	num		= data[4];
	ff		= data[5];

	led_index = -1;
}

TlnInfo::~TlnInfo()
{
}

const char* TlnInfo::GetTypName(unsigned int typ) const
{
	static const char* const maintyps[] = {"S0Amt","S0Intern","S0Quer","Analog","TFE"};
	const size_t maintypSize = sizeof(maintyps)/sizeof(char*);
	const char*  const maintypUnkown = "Typ nicht definiert";
	return((typ>=maintypSize) ? maintypUnkown : maintyps[typ] );
}

const char* TlnInfo::GetUTypName(unsigned int typ) const
{
	static const char* const subtyps[] = {"Telefon","Fax","Anrufbeantworter","Modem","Multifunktional","S0-Karte"};
	const size_t subtypSize = sizeof(subtyps)/sizeof(char*);
	const char*  const subtypUnkown = "Typ nicht definiert";
	return((typ>=subtypSize) ? subtypUnkown : subtyps[typ] );
}

int TlnInfo::BitPos() const
{
	int ret = (b1 << 8) | b2;
	return(ret);
}

int TlnInfo::Typ() const
{
	return((int)typ & 0x00FF);
}

int TlnInfo::Untertyp() const
{
	return((int)utyp & 0x00FF);
}

int TlnInfo::Num() const
{
	return((int)num & 0x00FF);
}

char *TlnInfo::Nummer() const
{
	static char ret[10];
	sprintf(ret,"%02X",num);
	return(ret);
}

ostream& operator<< ( ostream& strm , const TlnInfo& info) {

    strm << "Nummer: " << info.Nummer() << '\t';
    strm << "BitPos: " << info.BitPos() << '\t';
    strm << "NstTyp: " << info.GetTypName(info.Typ()) << '\t';
    strm << "UntTyp: " << info.GetUTypName(info.Untertyp()) << '\t';
    return strm;
}
