/* ******************************************************************
 * led.h
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: led.h,v 1.2 1999/12/16 08:08:39 root Exp $
 *
 * ******************************************************************/

/*
    This file was stolen from:

    File: led.h
    Version: 0.2.1
    Date: 1998/14/01
    Copyright: Bernhard Kuhn <kuhn@lpr.e-technik.tu-muenchen.de>

    This file defines an Led, which can be set to on or off :-)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    **********************
    ... and a little modified to fulfill my needs :)
*/



#ifndef LED_H
#define LED_H



#include <qframe.h>
#include <qdrawutl.h>



// define standard colors for enabled and disabled leds

#define LED_STD_OFFHOOK red
#define LED_STD_RINGING yellow
#define LED_STD_IDLE green
#define LED_STD_DEAD black



// define class for led-widget 

class led : public QFrame {

	// this class is an QOBJECT
	Q_OBJECT
  
public:

	enum State { Dead, OffHook, Ringing, Idle };

	// constructor for led-widget
	led(QWidget* parent=0, const char *name=0L);
    
	// change state of led
	void switchLed(State newstate=Dead);
	void dead() { switchLed(Dead); };
	void offhook() { switchLed(OffHook); };
	void ringing() { switchLed(Ringing); };
	void idle() { switchLed(Idle); };

protected:

	// what to do when widget should be repainted
	void drawContents(QPainter* p);
  
	// state of led
	State s;
  
	// pointer to current led brush
	QBrush ledBrush;
  
	// brush for states
	QColor dadColor, ohkColor, rngColor, idlColor;
    
public slots:

	// change state of led
	void setLed(State newstate=Dead);

};



#endif
