/********************************************************************
 * lib/Packet.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: Packet.c,v 1.2 1999/12/30 21:00:56 root Exp root $
 *
 ********************************************************************/
 
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>

#include <maKs.h>
#include <Packet.h>

EspMsg *allocEsp(size_t len)
{
	EspMsg *esp;

	if ((esp = calloc(sizeof(EspMsg) + len, 1)) == NULL) {
		perror("Allocating new ESP");
		exit(1);
	}

	return(esp);
}

EspMsg *createEsp(BYTE f, BYTE a, BYTE g, BYTE e, BYTE l, BYTE *d)
{
	EspMsg	*esp;
	int		len;

	len = l & 0x00FF;

	esp = allocEsp((size_t)len);

	esp->folgenr	=	f;
	esp->anlage		=	a;
	esp->gruppe		=	g;
	esp->element	=	e;
	esp->length		=	l;
	
	if (len)
		memcpy(esp->data, d, len);
	
	return(esp);
}

EspMsg *createBufEsp(BYTE *d, size_t len)
{
	EspMsg	*esp;
	
	esp = allocEsp((size_t)(len - 5));

	memcpy(&esp->folgenr, d, len);
	
	return(esp);
}

/*
 * deletes Packet
 */
void freeEsp(EspMsg *esp)
{
	if (esp == NULL) return;
	free(esp);
}

/*
 * Sends a Packet to given FD
 * returns int (answer)
 */
int sendPacket(int fd, EspMsg *esp)
{
	char c;
	int  ret;
	int len;

	len = 5 + ((int)esp->length & 0x00FF);

	ret = send(fd, esp, len, 0);

	if (ret != len) ret = -1;
	else ret = 0;

	if (0)
		analyse(esp,1,stderr);
	return(ret);
}

/*
 * Sends a Packet to given FD and deletes it afterwards
 * returns int (answer)
 */
int send_delPacket(int fd, EspMsg *esp)
{
	int ret;

	ret = sendPacket(fd, esp);

	free(esp);

	return(ret);
}

/*
 * Receives a Packet from given FD
 * returns Pointer to EPacket
 */
EspMsg *recv_Packet(int fd, int debug, FILE *logfp)
{
	EspMsg	*esp;
	BYTE	data[270];
	int		len;

	/* read 5 Byte MSGHeader */
	len = read(fd, data, 5);

	if (len <=0) {
		errno = ENOTCONN;
		return(NULL);
	}

	if (len != 5) fprintf(stderr,"Short Read!!!!!\n");

	len += read(fd, &data[5], ((int)data[4]) & 0x00FF);

	esp = allocEsp((size_t)(len - 5));

	memcpy(esp, data, len);
	
	if (0) analyse(esp, debug, logfp);
	return(esp);
}

/* Binaer ausgabe (geklaut bei Karsten Keil :) */
BYTE *prbits(BYTE *dest, BYTE b, int start, int len)
{
        BYTE *dp = dest;

        b = b << (8 - start);
        while (len--) {
                if (b & 0x80)
                        *dp++ = '1';
                else
                        *dp++ = '0';
                b = b << 1;
        }
        return (dest);
}

/*
 * Prints analysis of the packet.
 */
void analyse(EspMsg *esp, int debug, FILE *out)
{
	int	i;
	BYTE	tmpbuf1[10];
	BYTE	tmpbuf2[10];
	BYTE	tmpbuf3[10];
	BYTE	*chk_ok;
	BYTE	*cl;
	BYTE	*qa;
	BYTE	*rw;
	BYTE	*pt;
	BYTE	*sp;
	BYTE	*er;
	BYTE	*ad;


   	switch (esp->gruppe &0xF0) {

	case K_CTI_ESP:
		switch ((unsigned int)esp->element) {
			case ESP_CONN_REQ:
				cl="ESP_CONN_REQ"; break;
			case ESP_CONN_AUTH:
				cl="ESP_CONN_AUTH"; break;
			case ESP_SRVR_MSG:
				cl="ESP_SRVR_MSG"; break;
			default:
				cl="ESP_UNDEFINED";
				break;
		}
		break;

	case K_CTI_ANL:
		switch ((unsigned int)esp->element) {
			case 0x00: cl="ANL_LINK_ID_GET"; break;
			case 0x02: cl="ANL_TYP"; break;
			case 0x03: cl="ANL_SW_VERSION"; break;
			case 0x05: cl="ANL_BETRIEBSART"; break;
			case 0x0A: cl="ANL_MAX_PARA"; break;
			case 0x0C: cl="ANL_WERKSEINSTELLUNG"; break;
			case 0x0F: cl="ANL_FERNSCHALTEN"; break;
			case 0x11: cl="ANL_X31_KFG"; break;
			case 0x12: cl="ANL_GET_NST_TYP_NR"; break;
			case 0x13: cl="ANL_MASTER_SLAVE"; break;
			case 0x14: cl="ANL_KFG_START"; break;
			case 0x15: cl="ANL_KFG_ENDE"; break;
			case 0x16: cl="ANL_KFG_DATA_START"; break;
			case 0x17: cl="ANL_KFG_DATA_ENDE"; break;
			case 0x18: cl="ANL_KFG_DATA_ERROR"; break;
			case 0x19: cl="ANL_CRC_EPROM"; break;
			case 0x1A: cl="ANL_CRC_EEPROM"; break;
			case 0x1B: cl="ANL_FEHLERPROTOKOLL"; break;
			case 0x20:
				if (!(DODEBUG(debug,DBG_VERIFYLID))) return;
				cl="ANL_VERIFY_LINK_ID";
				break;
			default:
				cl="ANL_UNKNOWN";
		}
		break;

	case K_CTI_POR:
		switch ((unsigned int)esp->element) {
			case 0x00: cl="POR_LAND"; break;
			case 0x01: cl="POR_ORT"; break;
			case 0x02: cl="POR_HAUPTANLAGE"; break;
			case 0x04: cl="POR_MSN_AVA_TLN"; break;
			case 0x05: cl="POR_MSN_AVA_PROFIL"; break;
			case 0x06: cl="POR_MSN_AVA_SCHALTZEIT"; break;
			case 0x07: cl="POR_MSN_AVA_RWS_PROFIL"; break;
			case 0x08: cl="POR_MSN_AVA_RWS_ZEIT"; break;
			case 0x0A: cl="POR_AMT_RNR"; break;
			case 0x0B: cl="POR_INT_RNR"; break;
			case 0x0C: cl="POR_MODE"; break;
			case 0x0D: cl="POR_SER_DIENST"; break;
			case 0x0E: cl="POR_SER_RATE"; break;
			case 0x0F: cl="POR_DDI_LEN"; break;
			case 0x10: cl="POR_DDI_RNR"; break;
			case 0x11: cl="POR_TFE_AVA_TLN"; break;
			case 0x12: cl="POR_TFE_AVA_PROFIL"; break;
			case 0x13: cl="POR_TFE_AVA_SCHALTZEIT"; break;
			case 0x14: cl="POR_TFE_RUF_DAUER"; break;
			default:
				cl="POR_UNKNOWN";
		}
		break;

	case K_CTI_GEB:
		switch ((unsigned int)esp->element) {
			case 0x02: cl="GEB_PTT_SATZ"; break;
			case 0x0A: cl="GEB_ANL_STATUS"; break;
			case 0x0B: cl="GEB_TLN_INFO"; break;
			default:
				cl="GEB_UNKNOWN";
		}
		break;

	case K_CTI_ORG:
		switch ((unsigned int)esp->element) {
			case 0x02: cl="ORG_LOGO_FA"; break;
			case 0x03: cl="ORG_TLN_FA"; break;
			case 0x04: cl="ORG_TLN_GRP"; break;
			case 0x05: cl="ORG_RNR_FA"; break;
			case 0x07: cl="ORG_FA_STATUS"; break;
			case 0x08: cl="ORG_FA_CODE"; break;
			case 0x09: cl="ORG_FA_ANZAHL"; break;
			case 0x0A: cl="ORG_FA_MOH"; break;
			default:
				cl="ORG_UNKNOWN";
		}
		break;

	case K_CTI_TLN:
		switch ((unsigned int)esp->element) {
			case 0x00: cl="TLN_REGISTER"; break;
			case 0x18: cl="TLN_UMLEITUNG"; break;
			case 0x1A: cl="TLN_HOTLINE_RNR"; break;
			case 0x1D: cl="TLN_RECHTE"; break;
			case 0x1E: cl="TLN_SONSTIGES"; break;
			default:
				cl="TLN_UNKNOWN";
		}
		break;

	case K_CTI_AUX:
		switch ((unsigned int)esp->element) {
			case 0x00: cl="AUX_ALARM_AKTIV"; break;
			case 0x01: cl="AUX_ALARM_ZEIT"; break;
			case 0x02: cl="AUX_ALARM_DELAY"; break;
			case 0x03: cl="AUX_ALARM_DAUER"; break;
			case 0x04: cl="AUX_ALARM_RNR"; break;
			case 0x05: cl="AUX_ALARM_WAHL"; break;
			case 0x06: cl="AUX_ALARM_UUS"; break;
			case 0x08: cl="AUX_AKTOR_FKT"; break;
			case 0x09: cl="AUX_AKTOR_ZUSTAND"; break;
			case 0x0A: cl="AUX_AKTOR_MANUELL"; break;
			case 0x0B: cl="AUX_TLN_VIRTUELL"; break;
			case 0x0C: cl="AUX_PIN_VIRTUELL"; break;
			default:
				cl="AUX_UNKNOWN";
		}
		break;

	case UNKNOWN_7:	cl="07_UNDEFINED"; break;
	case UNKNOWN_8:	cl="08_UNDEFINED"; break;
	case UNKNOWN_9:	cl="09_UNDEFINED"; break;

	case K_CTI_STA:
		switch ((unsigned int)esp->element) {
			default:
				cl="STA_UNKNOWN";
		}
		break;

	case K_CTI_OFF:
		switch ((unsigned int)esp->element) {
			case 0x02: cl="OFF_REQUEST_GEB"; break;
			case 0x03: cl="OFF_TUEROEFFNER"; break;
			case 0x04: cl="OFF_ANL_ANRUFLISTE(IN)"; break;
			case 0x0B: cl="OFF_TLN_ANRUFLISTE(OUT)"; break;
			default:
				cl="OFF_UNKNOWN";
		}
		break;
	case K_CTI_VBG:
		switch ((unsigned int)esp->element) {
			case 0x02: cl="VBG_SETUP_AMT"; break;
			case 0x03: cl="VBG_SETUP_TLN"; break;
			case 0x04: cl="VBG_DISCONNECT"; break;
			case 0x05: cl="VBG_INTRUSION"; break;
			case 0x07: cl="VBG_U_CALL_TRANSFER"; break;
			case 0x0A: cl="VBG_3PARTY"; break;
			case 0x0C: cl="VBG_HOLD"; break;
			case 0x0D: cl="VBG_CONNECT"; break;
			case 0x20: cl="VBG_INT_CALL"; break;
			case 0x21: cl="VBG_EXT_CALL"; break;
			case 0x23: cl="VBG_GEB_INFO?"; break;
			case 0x24: cl="VBG_READY?"; break;
			case 0x28: cl="VBG_CONNECT?"; break;
			case 0x29: cl="VBG_READY?"; break;
			case 0x30:
				if (!(DODEBUG(debug,DBG_VERIFYLID))) return;
				cl="VBG_ANL_VERIFY_LINK_ID";
				break;
			default:
				cl="VBG_UNKNOWN";
		}
		break;

	case K_CTI_EVT:
		switch ((unsigned int)esp->element) {
			case 0x01: cl="EVT_HOOK_STATUS"; break;
			default:
				cl="EVT_UNKNOWN";
		}
		break;

	case K_CTI_TST:
		switch ((unsigned int)esp->element) {
			default:
				cl="TST_UNKNOWN";
		}
		break;

	case K_CTI_FAB:
		switch ((unsigned int)esp->element) {
			default:
				cl="FAB_UNKNOWN";
		}
		break;

	default:
		cl="IRGENDWAS_IST_FALSCH";
	}


	pt= (esp->folgenr == 0xFF) ? "BROADCAST, " : "";
	qa= (esp->gruppe & K_CTI_CR_MASK) ? "RSP" : "CMD"; 
	er= (esp->gruppe & K_CTI_OE_MASK) ? "ERR" : "OK"; 
	ad= (esp->gruppe & K_CTI_AD_MASK) ? "DEL" : "ADD"; 
	rw= (esp->gruppe & K_CTI_GS_MASK) ? "SET" : "GET"; 

	prbits(tmpbuf1,esp->gruppe,8,4);
	tmpbuf1[4]=0;

	prbits(tmpbuf2,esp->gruppe,4,4);
	tmpbuf2[4]=0;

	prbits(tmpbuf3,esp->element,8,8);
	tmpbuf3[8]=0;

	fprintf(out,"\n%s\n\t"
	      "PAKET#:   %02X       \tANLAGE: %02X\n\t"
	      "LAENGE:   %02X       \t\n\t"
	      "PAKETTYP: %02X (%4s)\tKOMMANDO: %02X (%8s)\n\t"
	      "FLAGS:    %02X (%4s) [%s %s %s %s]\t\t%s\n\t"
	      "DATA: ",
		  cl,
	      esp->folgenr, esp->anlage,
	      esp->length, 
	      (esp->gruppe & 0xF0), tmpbuf1, esp->element, tmpbuf3,
	      (esp->gruppe & 0x0F), tmpbuf2, qa,er,ad,rw,pt);	
	for (i=0;i<esp->length;i++) fprintf(out,"%02X ",esp->data[i]);
	fflush(out);
}
