/********************************************************************
 * setMaxSpeed/main.c
 * Part of maKs
 * Copyright Chris Benndorf, 1998
 * Released under GNU Public Library License
 * Best readeable with ":set tabstop=4"
 *
 * $Id: main.c,v 1.1 1999/12/30 21:05:49 root Exp root $
 *
 ********************************************************************/
 
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#include <version.h>
#include <config.h>

#include <maKs.h>
#include <Packet.h>
#include <Decode.h>
#include <server_conn.h>

#include "handle.h"

extern	int	done;
extern	int	s_port;
extern	int	s_speed;
		int	m_sock;
		int	debug = 0;

/********************************************************************
 *
 ********************************************************************/
void quit()
{
	done++;
	close(m_sock);
}

/*****************************************************************
 *
 *****************************************************************/
int parseSpeed(char *speed)
{
	if (!speed) return(3);

            if (!strcmp(speed,"B2400"))  return(0);
    else    if (!strcmp(speed,"B4800"))  return(1);
    else    if (!strcmp(speed,"B9600"))  return(2);
    else    if (!strcmp(speed,"B19200")) return(3);
    else    if (!strcmp(speed,"B38400")) return(4);
    else    if (!strcmp(speed,"2400"))   return(0);
    else    if (!strcmp(speed,"4800"))   return(1);
    else    if (!strcmp(speed,"9600"))   return(2);
    else    if (!strcmp(speed,"19200"))  return(3);
    else    if (!strcmp(speed,"38400"))  return(4);
    else    return(3);
}

/*****************************************************************
 *
 *****************************************************************/
char *prSpeed(int speed)
{
    switch (speed) {
        case 0:  return("2400");
        case 1:  return("4800");
        case 2:  return("9600");
        case 3:  return("19200");
        case 4:  return("38400");
        default: return("FEHLER");
    }
}

/********************************************************************
 *
 ********************************************************************/
void usage(char *prog)
{
    fprintf(stderr,"usage: %s [OPTIONS]\n"
        "\t[-s<PBXPORT>] 0=Configport (default) 1=Printerport\n"
        "\t[-S<SPEED>]   serial speed for pbx port (2400,4800,9600,19200)\n"
        "\t[-u<USER>]    Username\n"
        "\t[-p<PASS>]    Password\n"
        "\t[-H<Host>]    Host that runs Timm\n"
        "\t[-P<Port>]    Port that Timm is running on\n"
        "\t[-u,-p,-H,-P] do overwrite ~/.maKsrc\n"
        "\t%s\n",prog,rcsid);
    exit(1);
} 

/********************************************************************
 *
 ********************************************************************/
int main(int argc, char **argv)
{
    extern char *optarg;
    extern int optind;
	char			*c_speed = NULL;
	char			*c_port  = NULL;
    char			*machine = "localhost";
    char			*user    = getenv("LOGNAME");
    char			*passwd  = NULL;
    int				 port=TCPPORT;
    int c;

	signal(SIGINT,quit);
	signal(SIGHUP,quit);
	signal(SIGTERM,quit);


	read_maKsrc(&user, &passwd, &machine, &port); 

#define OPTSTRING   "H:P:u:p:S:s:h"

	while ((c=getopt(argc,argv,OPTSTRING)) != EOF)
		switch(c) {
			case 'H':
				machine=optarg;
				break;
			case 'P':
				port=atoi(optarg);
				break;
			case 'u':
				user=optarg;
				break;
			case 'p':
				passwd=optarg;
				break;
			case 'S':
				c_speed=optarg;
				s_speed=parseSpeed(c_speed);
				break;
			case 's':
				s_port=atoi(optarg);
				if (s_port > 1) s_port=0;
				break;
			default:
				usage(argv[0]);
		}

	if (!set_user(user)) usage(argv[0]);
	if (!set_passwd(passwd)) usage(argv[0]);

	InitProcs(0, stderr);

	/* Want to handle Login myself */
	SetUsrProc(K_CTI_ESP, &handle_CTI_ESP);

	/* Need to know if ANL_KFG_START is ok */
	SetUsrProc(K_CTI_ANL, &handle_CTI_ANL);

	/* Need to know if POR_SER_RATE is ok */
	SetUsrProc(K_CTI_POR, &handle_CTI_POR);

	if((m_sock=Server_Connect(machine,port))== -1)
		exit(1);

	conn_request(m_sock, PERM_KONFIG, AUTH_CLEARTXT, "setMaxSpeed", ASYNC);

	printf("\nKonfiguriere %s-Port auf %s Baud\n\n",
			(s_port) ? "Drucker" : "Konfig",
			prSpeed(s_speed));
	if (!s_port)
		printf(	"Nach der Konfiguration muss Timm gestoppt und mit anderer\n"
				"Port-Geschwindigkeit neu gestartet werden!\n"
				"Nach Beendigung der Umkonfiguration muss dieses Programm\n"
				"hoechstwahrscheinlich mit ^C abgebrochen werden, da die\n"
				"Anlage sofort umschaltet und Timm nicht mehr korrekt empfaengt.\n\n");
	PacketWaitAsync(m_sock);


	return(0);
}
